# $Idaemons: /home/cvs/pkgtools/pkgdb.rb,v 1.12 2001/03/22 19:04:36 knu Exp $

class PkgDB
  DB_DIR = ENV['PKG_DBDIR'] || '/var/db/pkg'

  PKGDB_FILES = {
    :contents => '+CONTENTS',
    :comment => '+COMMENT',
    :desc => '+DESC',
    :install => '+INSTALL',
    :post_install => '+POST-INSTALL',
    :deinstall => '+DEINSTALL',
    :post_deinstall => '+POST-DEINSTALL',
    :require => '+REQUIRE',
    :required_by => '+REQUIRED_BY',
    :display => '+DISPLAY',
    :mtree => '+MTREE_DIRS',
  }

  attr_reader :db_dir

  def initialize(db_dir = nil)
    @db_dir = db_dir || DB_DIR
  end

  def strip(pkgdir)
    pkgname = File.basename(pkgdir)

    if installed?(pkgname)
      return pkgname
    end

    nil
  end

  def pkgdir(pkgname)
    File.join(@db_dir, pkgname)
  end

  def pkgfile(pkgname, filename)
    if filename.kind_of?(Symbol)
      filename = PKGDB_FILES[filename]
    end

    File.join(pkgdir(pkgname), filename)
  end

  def installed?(pkgname)
    File.exist?(pkg_comment(pkgname))
  end

  def required?(pkgname)
    File.exist?(pkg_required_by(pkgname))
  end

#  def really_required?(pkgname)
#  end

  def required_by(pkgname)
    filename = pkg_required_by(pkgname)

    File.exist?(filename) or return nil

    return File.open(filename).readlines.collect {
      |line| line.chomp
    }
  end

  PKGDB_FILES.each_key do
    |key|
    module_eval %{
      def pkg_#{key.to_s}(pkgname)
	pkgfile(pkgname, #{key.inspect})
      end
    }
  end

  def glob(glob = '*')
    # ugly hack until we have String#fnmatch()..
    re = glob.gsub(/([.+])/, "\\\\\\1").tr("?", ".").gsub(/\*/, ".*")
    re = /^#{re}(-[^-]*)?$/

    list = []

    Dir.entries(@db_dir).each do
      |pkgname|
      next unless installed?(pkgname)

      re =~ pkgname or next

      begin
	pkg = PkgInfo.new(pkgname)

	if block_given?
	  yield pkg
	else
	  list.push pkg
	end
      rescue => e
	STDERR.puts e.message + ": skipping..."
      end
    end

    if block_given?
      nil
    else
      list
    end
  end
end
