# $Idaemons: /home/cvs/pkgtools/pkginfo.rb,v 1.4 2001/02/13 09:12:25 knu Exp $

require 'pkgversion'

class PkgInfo
  include Comparable

  PKG_INFO_CMD = 'pkg_info'
  PKG_INFO_FLAGS = {
    :prefix => 'p',
    :comment => 'c',
    :descr => 'd',
    :message => 'D',
    :plist => 'f',
    :install => 'i',
    :deinstall => 'k',
    :req => 'r',
    :required_by => 'R',
    :mtree => 'm',
    :files => 'L',
    :totalsize => 's',
    :origin => 'o',
  }

  attr_accessor :name, :version

  def initialize(pkgname)
    if /\s/ =~ pkgname
      raise ArgumentError, "#{pkgname}: Must not contain whitespace."
    end

    unless /^(.+)-([^-]+)$/ =~ pkgname
      raise ArgumentError, "#{pkgname}: Not in due form: <name>-<version>"
    end

    @name = $1
    @version = PkgVersion.new($2)
  end

  def to_s
    @name + '-' + @version.to_s
  end

  alias fullname to_s

  def coerce(other)
    case other
    when PkgInfo
      return other, self
    when PkgVersion
      return other, @version
    when String
      if /-/ =~ other
	return PkgInfo.new(other), self
      else
	return PkgVersion.new(other), @version
      end
    else
      raise TypeError, "Coercion between #{other.type} and #{self.type} is not supported."
    end
  end

  def <=>(other)
    other_name = nil

    case other
    when PkgInfo
      other_name = other.name
      other_version = other.version
    when PkgVersion
      other_version = other
    when String
      if /-/ =~ other
	other = PkgInfo.new(other)
	other_name = other.name
	other_version = other.version
      else
	other_version = PkgVersion.new(other)
      end
    else
      a, b = other.coerce(self)

      return a <=> b
    end
    if other_name
      cmp = @name <=> other_name
      return cmp if cmp != 0
    end

    @version <=> other_version
  end

  def self.get_info(pkg, what, pkg_info_cmd = PKG_INFO_CMD)
    opt = PKG_INFO_FLAGS[what]

    if opt == nil
      raise ArgumentError, "#{what.to_s}: Unsupported information."
    end

    info = `#{pkg_info_cmd} -q#{opt} #{pkg} 2>/dev/null`.chomp

    info == '' ? nil : info
  end

  def get_info(what)
    PkgInfo::get_info(fullname(), what)
  end

  PKG_INFO_FLAGS.each_key do
    |key|
    module_eval %`
    def #{key.to_s}
      get_info(#{key.inspect})
    end
    `
  end
end

class PkgFileInfo < PkgInfo
  def initialize(pkgfilename)
    info = PkgInfo::get_info(pkgfilename, :plist)

    if info.nil?
      raise ArgumentError, "#{pkgfilename}: Couldn't get package info."
    end

    info.each_line do |line|
      if /^@name\s+(\S+)$/ =~ line
	return super($1)
      end
    end

    raise ArgumentError, "#{pkgfilename}: Couldn't get package name."
  end
end
