# $Idaemons: /home/cvs/pkgtools/portinfo.rb,v 1.1 2001/02/13 12:48:37 knu Exp $

require 'pkginfo'

class PortInfo
  FIELDS = [ :pkgname, :origin, :prefix, :comment, :descr_file,
             :maintainer, :categories, :build_depends, :run_depends, :www ]
  LIST_FIELDS = [ :categories, :build_depends, :run_depends ]
  PORTS_DIR_FIELDS = [ :origin, :descr_file ]
  NFIELDS = FIELDS.size
  FIELD_SEPARATOR = '|'

  def initialize(line)
    unless line.is_a?(String)
      raise ArgumentError, "You must specify a one line text of port info."
    end

    values = line.chomp.split(FIELD_SEPARATOR, NFIELDS)

    if values.size != NFIELDS || values[NFIELDS - 1].index(FIELD_SEPARATOR)
      raise ArgumentError, "Port info line must consist of #{NFIELDS} fields."
    end

    @attr = {}

    ports_dir = nil

    FIELDS.each_with_index do |field, i|
      value = values[i]
      if value.nil? || value.empty?
	@attr[field] = nil
      else
	case field
	when :pkgname
	  value = PkgInfo.new(value)
	when :origin
	  value.sub! %r`^(.*)/([^/]+/[^/]+)$`, '\\2'
	  ports_dir = $1
	when :descr_file
	  value.sub! %r`^#{Regexp.quote(ports_dir)}/`, ''
	when *LIST_FIELDS
	  value = value.split ' '
	end

	@attr[field] = value
      end
    end
  end
  
  FIELDS.each do |field|
    module_eval %`
    def #{field.to_s}
      @attr[#{field.inspect}]
    end
    `
  end

  def to_s(ports_dir = PortsDB::PORTS_DIR)
    FIELDS.collect { |field|
      value = @attr[field]
      if value.nil?
	''
      else
	case field
	when :pkgname
	  value = value.to_s
	when :origin, :descr_file
	  value = File.join(ports_dir, value)
	when *LIST_FIELDS
	  value.join ' '
	else
	  value
	end
      end
    }.join(FIELD_SEPARATOR) + "\n"
  end
end
