# $Idaemons: /home/cvs/pkgtools/portsdb.rb,v 1.2 2001/02/13 20:18:27 knu Exp $

require 'dbm'

class PortsDB
  include Enumerable

  PORTS_DIR = ENV['PORTSDIR'] || '/usr/ports'

  attr_reader :ports_dir, :index_file, :db_dir, :db_file

  def initialize(db_dir = nil, ports_dir = nil)
    @ports_dir = ports_dir || PORTS_DIR
    @index_file = File.join(@ports_dir, 'INDEX')
    @db_dir = db_dir || @ports_dir
    @db_file = File.join(@db_dir, 'INDEX.db')
    @db_filebase = @db_file.sub(/\.db$/, '')
    @categories = nil
    @ports = {}
    @dbm = nil
  end

  def categories
    if @categories
      return @categories
    end

    @categories = %x`cd #{@ports_dir} && make -V SUBDIR`.strip.split(' ')
  end

  def update(make_args = "-j4")
    system "cd #{@ports_dir} && make #{make_args} index"
    @categories = nil
    @ports = {}
    @dbm = nil

    update_db
  end

  def open_db
    unless @dbm
      if !File.exist?(@db_file) || File.stat(@db_file) < File.stat(@index_file)
	update_db
      else
	@dbm = DBM.open(@db_filebase)
      end
    end

    @dbm
  end

  def update_db
    if @dbm
      @dbm.close
    end

    begin
      @dbm = DBM.open(@db_filebase, 0664)

      @dbm.clear

      File.open(@index_file) do |f|
	f.each do |line|
	  port_info = PortInfo.new(line)
	  @dbm[port_info.origin] = port_info.to_s
	end
      end
    rescue => e
      puts "#{e.message}: Cannot update the ports DB!"
    end

    @dbm
  end

  def port(origin)
    if @ports[origin]
      return @ports[origin]
    end

    open_db

    line = @dbm[origin]

    @ports[origin] = if line then PortInfo.new(line) else nil end
  end

  alias [] port

  def each
    open_db

    @dbm.each { |*args| yield(*args) }
  end

  def each_port
    open_db

    @dbm.each_key { |origin| yield(origin) }
  end
end
